@@ -3,6 +3,7 @@ This module is responsible for extracting and parsing configuration from Cobalt
 """
 import os
 import io
+import sys
 import hashlib
 import logging
 import ipaddress
@@ -132,9 +133,13 @@ enum BeaconSetting: uint16 {
     SETTING_MAX_RETRY_STRATEGY_ATTEMPTS = 71,
     SETTING_MAX_RETRY_STRATEGY_INCREASE = 72,
     SETTING_MAX_RETRY_STRATEGY_DURATION = 73,
+
+    // CobaltStrike version >= 4.7 (Aug 17, 2022)
+    SETTING_MASKED_WATERMARK = 74,
 };
 
 enum DeprecatedBeaconSetting: uint16 {
+    SETTING_KILLDATE_YEAR = 16,
     SETTING_INJECT_OPTIONS = 36,
 };
 
@@ -578,6 +583,7 @@ SETTING_TO_PRETTYFUNC: Dict[BeaconSetting, Callable] = {
     BeaconSetting.SETTING_DNSRESOLVER: null_terminated_str,
     BeaconSetting.SETTING_DNS_IDLE: lambda x: str(ipaddress.IPv4Address(x)),
     BeaconSetting.SETTING_WATERMARKHASH: lambda x: null_terminated_bytes(x) if isinstance(x, bytes) else x,
+    BeaconSetting.SETTING_MASKED_WATERMARK: lambda x: x.hex()
     # BeaconSetting.SETTING_PROTOCOL: lambda x: BeaconProtocol(x).name,
     # BeaconSetting.SETTING_CRYPTO_SCHEME: lambda x: CryptoScheme(x).name,
     # BeaconSetting.SETTING_PROXY_BEHAVIOR: lambda x: ProxyServer(x).name,
@@ -844,7 +850,7 @@ class BeaconConfig:
         domains = self.raw_settings.get("SETTING_DOMAINS")
         if not isinstance(domains, bytes):
             return []
-        return [pair for pair in grouper(null_terminated_str(domains).split(","), 2)]
+        return list(grouper(null_terminated_str(domains).split(","), 2))
 
     @property
     def uris(self) -> List[str]:
@@ -972,11 +978,16 @@ def main():
 
     xor_keys = None
     if args.xorkey:
-        xor_keys = tuple(utils.pack(int(x, 0)) for x in args.xorkey)
+        xor_keys = tuple(utils.pack_be(int(x, 0)) for x in args.xorkey)
 
-    config = BeaconConfig.from_path(args.input, xor_keys=xor_keys, all_xor_keys=args.all)
-    if not config:
-        print("Beacon configuration not found.")
+    try:
+        if args.input in ("-", "/dev/stdin"):
+            with io.BytesIO(sys.stdin.buffer.read()) as fin:
+                config = BeaconConfig.from_file(fin, xor_keys=xor_keys, all_xor_keys=args.all)
+        else:
+            config = BeaconConfig.from_path(args.input, xor_keys=xor_keys, all_xor_keys=args.all)
+    except ValueError:
+        print(f"{args.input}: No beacon configuration found.", file=sys.stderr)
         return 1
 
     if args.type == "raw":
@@ -998,4 +1009,4 @@ def main():
 
 
 if __name__ == "__main__":
-    main()
+    sys.exit(main())
